# copybook_main.pl - Top script called to upload copybook to mainframe 
#
# Authors:  Jane Huang
# Last updated:  3/22/07
#								 Xiao Lin Zhang, 7/06/2009
#
# (c) Copyright IBM Corp. 2008
#
# The common routines used in all the build scripts are contained in the 
# bccpbmod.pm module.
#
# This script is the top of the call sequence for uploading copybook member
# to mainframe.
#
# You must modify this script before attempting to use parameterized build.
# Sections that require modification and the instructions for modifying them
# are flagged with the following comment lines:
#
# ******************************************************************************
# * Customization instructions:                                                *
# * [instructions here]                                                        *
# ******************************************************************************

use File::Basename;
use File::Copy;
use Getopt::Std;
use bccpbmod;

my $debug = 1;

# #############################################################################
# Use getopt to process the command arguments
#
# "-i profileUSER_Path" identifies the directory where the profileUSER are located
# "-d dir"   identifies the directory where the elements are located
# "file ..." identifies the file or list of files to be uploaded
#

my $usage = "copybook_main.pl -i profileUSER_Path -d dir file ...";

my $upload_1 = "";
my $elem_dir  = ""; # directory where the file resides

  if (!getopts('i:d:') || $opt_h) {
    print "$usage \n";
    exit(0);
}

$elem_dir  = $opt_d;
$Path_profileUSER = $opt_i;

# *******************************************************************************
# * Required customization:                                                     *
# * Judge OS type to determine the delimiter embedded in path, you can modify it*
# * according to you site's requirement or you can hardcode delimiter here.     *
# ******************************************************************************* 
my $delimiter = '';
my $OS = $^O;
if ($OS =~/.*Win.*/){	#For Windows operating system
	$delimiter = "\\";
	$current_dir = `cd`;
}elsif ($OS =~/.*n.*x/ or $OS eq "hpux" or $OS eq "solaris"){	# For Unix & Linux operating system
	$delimiter = "/";
$current_dir = `pwd`;
}elsif ($OS eq "aix"){	#For AIX operating system
	$delimiter = "/";
$current_dir = `pwd`;
}
chomp $current_dir;

#diagnose if the path has a tail delimiter
$Size = length($elem_dir);
$position = rindex($elem_dir,$delimiter);
if($position eq ($Size-1)){
	chop $elem_dir;# delete the tail delimiter for further processing
}

print "Main Copybook: current dir =  $current_dir \n" if $debug;
print "Main Copybook: dir_file = $elem_dir \n" if $debug;

# *******************************************************************************
# * Required customization:                                                     *
# * The location of the build script can be get in profileUSER									*
# *******************************************************************************

my $scriptdir = "";   # location of the build scripts
my $uploadcopy= "";   # Upload CopyBook script 

($scriptdir,$lifecycle,$EDfile,$VIEWfile,$listdir,$logdir,$perlexe) = bccpbmod::getprofileUSER($Path_profileUSER,$delimiter);

    if ($scriptdir) {
        
        $uploadcopy = $scriptdir . $delimiter . "uploadCopybook.pl";         
            
        printf( "Build script dir by attribute SCRIPTDIR:(%s)\n", $scriptdir ) if $debug; 
        printf( "Upload Copybook script:(%s)\n", $uploadcopy ) if $debug;  

    }     
    
    # check if $uploadcopy = "";
    if ($scriptdir eq "" || $uploadcopy eq "") {
      	print " Define the build script directory and upload copybook script !\n" ;
   	bccpbmod::pause();
        exit(0);
    }
  

print "Current Directory " .$elem_dir. "\n\n";

# #############################################################################
# check for multiple files to upload and set a variable to be used for
# the upload scripts
# 
my $num_args = @ARGV;
my $upload_1 = "";
$upload_1 = "-1" if ($num_args == 1);


# #############################################################################
# Get the environment variables from the common module
# These variables establish the distributed and mainframe environment
# and must be modified for each installation.  The values for these 
# variables are held in projectED file.
# These varibles are used to point to the Z/OS infrastructure/environment/mapped 
# Life cycle that the project will take. This includes SYSLIB concatenation to be
# used for each environment ie: TEST, QA, PROD. What target loadlibs are to be used
# DB2 subsystem, CICS region, IMS region etc. They are declared at the project level
# the are queried and used here.
# *******************************************************************************
# * Optional customization:                                                     *
# * You can use the optional variables or add new variables based on your site's*
# * needs.                                                                      *
# *******************************************************************************
my ($env, $db2sys, $os390_hostname, $os390_port, $stage,
$copylib1, $copylib2, $copylib3, $copylib4,
$syslib1, $syslib2, $syslib3,
#$syslib4, $steplib1, $steplib2,			 # optional
#$hostpds_prefix, $hostpds_affix, $cbllib,	         # optional 
$loadlib, $c390llib, $dbrmlib, $tgtlib, $db2_owner) = bccpbmod::setEnvironmentVar($EDfile,$lifecycle,$delimiter);
    
# print "Env: $env \n" if $debug;
# print "db2sys: $db2sys \n" if $debug;


# #############################################################################
# Check for connectivity to the mainframe and die if not successful.
# 

bccpbmod::my_die("Server down error.") if (bccpbmod::testServer($os390_hostname, $os390_port) != 0);

# #############################################################################
# Identify each of the file types and separate files from the arguments list
# into arrays by type.  These arrays are passed to the type specific uploading
# scripts.
# 

my @cobol2_files;
my $no_cobol2_files = 0;

my $no_unsupported_files = 0;
my $message ="";  # test only -add


foreach $path (@ARGV) {
	my %attrHash = bccpbmod::getAttrs($VIEWfile,$path);

	print "path=$path LANG=$attrHash{'LANG'}\n" if $debug;

	push(@cobol2_files, $path);

}
 
# #############################################################################
# If any of the file type arrays contain filenames, then execute the uploading
# script for each file in the list.
# 

$no_cobol2_files = @cobol2_files;


# ------------------------- Upload CopyBook files -----------------------------
if ($no_cobol2_files != 0) {
	print "Uploading Copybook files\n";
	bccpbmod::printArray(@cobol2_files);

	foreach $path (@cobol2_files) { 
		my($file, $directory, $suffix) = fileparse($path);   
       		
		print "======================================================= \n";
		print " Uploading file: " .$file. "\n";
    

# #############################################################################  		
# Exit if the file is not a COPYBOOK     
#
    
   my($testfile,$testdir,$ext) = File::Basename::fileparse($file,qr{.cpy});  
     
   if (($ext ne ".cpy") && ($ext ne ".CPY")) {
      	print "ERROR- File extension $ext is not correct !\n" ;
 	bccpbmod::pause();
	exit -1;
   }

# #############################################################################
# Upload the copybook file
#

# ############################################################################
# Cleanup list and log files from the previous execution
		bccpbmod::cleanup($logdir,$delimiter);
		bccpbmod::cleanup($listdir,$delimiter);

		my $logfile = "";
		$logfile = $logdir . $delimiter . $file . "_uploadCopybook.log";
   	print $logfile . "\n";

	  print `\"$perlexe\" -I \"$scriptdir\" \"$uploadcopy\" $upload_1 -i \"$Path_profileUSER\" -d \"$elem_dir\" -f $file > \"$logfile\"`;
         
                  
# #############################################################################
# move rccbuild.log to user-defined log dir
#
if(-f $elem_dir . $delimiter . "rccbuild.log"){
	move($elem_dir . $delimiter . "rccbuild.log",$logdir . $delimiter . "rccbuild.log");
}
unlink("." . $delimiter."rccbuild.log");
# #############################################################################
# move new output listing file to user-defined listing dir
#
bccpbmod::movelist($elem_dir,$listdir,$delimiter);
    
    
		if ($?) {
			bccpbmod::pause();
			exit -1;
		}

		print "\n-------------------------------------------------------- \n";
		print "Completed uploading copybook member of $file\n";
		print "-------------------------------------------------------- \n\n";
	}
}


#------------------------------------------------------
#
# Wait at the end of the upload process for the user to press "any key".  This allows the user to
# review the DOS window output before the window is closed.
bccpbmod::pause();

exit 0;

#######################################################
